@@ -54,10 +54,12 @@ class Stater:
         rec( [self.tag._getTree()])
         logger.debug("Stater.__init__(), save %s states", len(self._states))
 
-    def guess(self,dontRedraw=Optional[Tag]): # -> list<Tag>
+    def guess(self): # -> list<Tag>
         """ to be runned after interactions to guess whose are modifieds
             return modifieds tags
         """
+        #TODO: the 'dontRedraw' shouldn't be set since htag>=0.3.0, so this code could be redsigned
+
         logger.debug("Stater.guess(), start guessing ....")
         modifieds=[]
 
@@ -66,15 +68,10 @@ class Stater:
                 tag,childs= list(obj.items())[0]
                 state_after = tag._getStateImage()
                 state_before= self._states.get( id(tag) )
-                if state_after != state_before:
-                    if id(tag)==id(dontRedraw):
-                        logger.debug("Don't redraw %s (but modified)", repr(dontRedraw))
-                        # but perhaps childs ...
-                        rec(childs)
-                    else:
-                        logger.debug("STATE BEFORE for %s = '%s'", repr(tag), state_before )
-                        logger.debug("STATE AFTER  for %s = '%s'", repr(tag), state_after )
-                        modifieds.append(tag)
+                if state_after != state_before and isinstance(tag,Tag):
+                    logger.debug("STATE BEFORE for %s = '%s'", repr(tag), state_before )
+                    logger.debug("STATE AFTER  for %s = '%s'", repr(tag), state_after )
+                    modifieds.append(tag)
                 else:
                     # no need to see childs, coz the parent will redraw all (childs too)
                     rec(childs)
@@ -85,17 +82,31 @@ class Stater:
 
 
 class HRenderer:
-    def __init__(self, tag: Tag, js:str, exit_callback:Optional[Callable]=None):
+    def __init__(self, tagClass: type, js:str, exit_callback:Optional[Callable]=None, init= ((),{}), fullerror=False, statics=[] ):
         """ tag object will be setted as a 'body' tag ! """
         """ js should containt an interact method, and an initializer to call the start()"""
-        if not isinstance(tag, Tag): raise HTagException("Can only render a Tag !")
+        if not issubclass(tagClass, Tag): raise HTagException("HRenderer can only handle tag subclasses !")
+        if not isinstance(statics, list): raise HTagException("HRenderer statics should be a list !")
+        self.fullerror = fullerror
+        self._interaction_scripts=[]
+        self.init = tuple( [tuple(init[0]),dict(init[1])] ) # save args/kargs whose initialized the instance
+
+        try:
+            args,kargs = init
+            kargs["hr"] = self
+            tag = tagClass( *args,**kargs )
+        except TypeError:
+            logger.warning(f"Can't instanciate tag '{tagClass.__name__}' with {init} arguments, so instanciate it without argument !")
+            kargs={"hr": self}
+            tag = tagClass(**kargs)
+
         self.tag=tag
         self.tag.tag="body" # force first tag as body !!
         if exit_callback:
             # add an .exit() method on the tag !!!!!!!!!!!!!!!!!!!!!!
             self.tag.exit = exit_callback
 
-        self._statics: list = []
+        self._statics: list = list(statics)
 
         ensureList=lambda x: list(x) if type(x) in [list,tuple] else [x]
 
@@ -103,20 +114,23 @@ class HRenderer:
             for i in ensureList(tag.statics):
                 if isinstance(i,TagBase):
                     if isinstance(i,Tag):
-                        logger.warning("Avoid to include dynamic Tag in statics! (it will be converted)")
                         i = i._ensureTagBase()
 
                     if i.md5 not in [j.md5 for j in self._statics]:
                         self._statics.append( i )
-                # else:
-                #     raise HTagException("Statics must be a TagBase/H")
+                elif isinstance(i,str): # auto add as Tag.style // CSS
+                    self._statics.append( Tag.H.style(i))
+                elif isinstance(i,bytes): # auto add as Tag.script // JS
+                    self._statics.append( Tag.H.script(i.decode()))
+                else:
+                    raise HTagException("Included static is bad")
 
 
         if hasattr(self.tag, "imports") and self.tag.imports is not None:
             # there is an "imports" attribut
             # so, try to import statics according "imports" attrs on tags
             logger.info("Include statics from Tag's imports attibut")
-            feedStatics(tag)
+            feedStatics(self.tag)
             def rec( tag ):
                 if hasattr(tag, "imports") and tag.imports is not None:
                     imports = ensureList(tag.imports)
@@ -143,6 +157,13 @@ class HRenderer:
         js_base="""
 function start() { %s }
 
+function _error(txt,env) {
+    if(window.error)
+        error( env+" ERROR: "+txt );
+    else
+        console.log( env+" ERROR:", txt );
+}
+
 function action( o ) {
 
     if(o.hasOwnProperty("update"))
@@ -154,9 +175,9 @@ function action( o ) {
             document.getElementById( key ).innerHTML += o["stream"][key];
         });
 
-    if(o.hasOwnProperty("post")) eval( o["post"] );
-    if(o.hasOwnProperty("next")) eval( o["next"] );
-    if(o.hasOwnProperty("err"))  console.log( "ERROR", o["err"] );
+    if(o.hasOwnProperty("post")) {try{ eval( o["post"] )} catch(e) {_error(e, "JS");throw e}};
+    if(o.hasOwnProperty("next")) {try{ eval( o["next"] )} catch(e) {_error(e, "JS");throw e}};
+    if(o.hasOwnProperty("err")) _error( o["err"], "PYTHON")
 }
 
 %s
@@ -166,10 +187,14 @@ function action( o ) {
 
         self._loop={}   # for savng generator, to keep them during GC
 
+
+    def _addInteractionScript(self, js:str):
+        self._interaction_scripts.append(js)
+
     async def interact(self,oid,method_name:str,args,kargs) -> dict:
         """ call the 'method_name' of pyobj 'id' with (args,kargs), return the pyobj/tag"""
         try:
-            interaction_scripts=[] # interact js scripts
+            ## self._interaction_scripts=[] # reset the list
             next_js_call=None
 
             if oid==0:
@@ -178,86 +203,71 @@ function action( o ) {
                 rep = self._mkReponse( [self.tag] )
                 rep["update"]={0: list(rep["update"].values())[0]}  #INPERSONNATE (for first interact on id#0)
             else:
-                def hookInteractScripts(obj,js:str):
-                    interaction_scripts.append( obj._genIIFEScript(js) ) #IIFE !
 
                 state = Stater(self.tag)
-                setattr(Tag,"__call__", hookInteractScripts) # not great (with concurrencies)
-                try:
-                    norender=False
+                obj = Tag.find_tag(oid)
+                if obj:
+                    # call the method
+                    method=getattr(obj,method_name)
 
-                    obj = Tag.find_tag(oid)
-                    if obj:
-                        # call the method
-                        method=getattr(obj,method_name)
-
-                        norender = hasattr(method,"_norender")
-
-                        logger.info(f"INTERACT with METHOD {fmtcaller(method_name,args,kargs)} %s, of %s", "**NoRender**" if norender else "", repr(obj) )
-
-                        if asyncio.iscoroutinefunction( method ):
-                            r=await method(*args,**kargs)
-                        else:
-                            r=method(*args,**kargs)
-
-                        if isinstance(r, types.AsyncGeneratorType) or isinstance(r, types.GeneratorType):
-                            # save it, to avoid GC (and save the Tag/norender)
-                            self._loop[id(r)] = dict(gen=r,tag=obj,norender= norender)
+                    logger.info(f"INTERACT with METHOD {fmtcaller(method_name,args,kargs)}, of %s", repr(obj) )
 
+                    if asyncio.iscoroutinefunction( method ):
+                        r=await method(*args,**kargs)
                     else:
-                        obj = self._loop.get(oid,None)
-                        if obj: # it's a existing generator
-                            r, obj, norender = obj["gen"], obj["tag"], obj["norender"]
-                            logger.info("INTERACT with GENERATOR %s(...) %s, of %s",r.__name__, "**NoRender**" if norender else "", repr(obj) )
-                        else:
-                            raise Exception(f"{oid} is not an existing Tag or generator (dead objects ?)?!")
+                        r=method(*args,**kargs)
 
+                    if isinstance(r, types.AsyncGeneratorType) or isinstance(r, types.GeneratorType):
+                        # save it, to avoid GC
+                        self._loop[id(r)] = dict(gen=r,tag=obj)
+
+                else:
+                    obj = self._loop.get(oid,None)
+                    if obj: # it's a existing generator
+                        r, obj = obj["gen"], obj["tag"]
+                        logger.info("INTERACT with GENERATOR %s(...), of %s",r.__name__, repr(obj) )
+                    else:
+                        raise Exception(f"{oid} is not an existing Tag or generator (dead objects ?)?!")
+
+                ret=None
+                if isinstance(r, types.AsyncGeneratorType):
+                    # it's a "async def yield"
+                    try:
+                        ret = await r.__anext__()
+                        next_js_call = BaseCaller(r)
+                    except StopAsyncIteration:
+                        del self._loop[ id(r) ]
+                elif isinstance(r, types.GeneratorType):
+                    # it's a "def yield"
+                    try:
+                        ret = r.__next__()
+                        next_js_call = BaseCaller(r)
+                    except StopIteration:
+                        del self._loop[ id(r) ]
+                else:
+                    # it's a simple method
+                    assert r is None
                     ret=None
-                    if isinstance(r, types.AsyncGeneratorType):
-                        # it's a "async def yield"
-                        try:
-                            ret = await r.__anext__()
-                            next_js_call = BaseCaller(r)
-                        except StopAsyncIteration:
-                            del self._loop[ id(r) ]
-                    elif isinstance(r, types.GeneratorType):
-                        # it's a "def yield"
-                        try:
-                            ret = r.__next__()
-                            next_js_call = BaseCaller(r)
-                        except StopIteration:
-                            del self._loop[ id(r) ]
+
+                rep= self._mkReponse(state.guess() )
+
+                if ret:
+                    obj.add(ret) # add content in object
+                    if not isinstance(ret,str) and hasattr(ret,"__iter__"):
+                        rep.setdefault("stream",{})[ id(obj) ] = "".join([str(i) for i in ret])
                     else:
-                        # it's a simple method
-                        assert r is None
-                        ret=None
+                        rep.setdefault("stream",{})[ id(obj) ] = str(ret)
 
-                    if norender:
-                        rep= self._mkReponse(state.guess(obj) ) # avoid redraw obj
-                    else:
-                        rep= self._mkReponse(state.guess() )
-
-                    if ret:
-                        obj.add(ret) # add content in object
-                        if not isinstance(ret,str) and hasattr(ret,"__iter__"):
-                            rep.setdefault("stream",{})[ id(obj) ] = "".join([str(i) for i in ret])
-                        else:
-                            rep.setdefault("stream",{})[ id(obj) ] = str(ret)
-
-                finally:
-                    # clean the (fucking) situation ;-)
-                    try: # to avoid del trouble in multi concurency
-                        del Tag.__call__
-                    except AttributeError:
-                        pass
 
             # --> rep
-            if interaction_scripts:
+            if self._interaction_scripts:
                 # some js scripts was generated during interactions
                 if "post" not in rep:
                     rep["post"]=""
                 # add them
-                rep["post"] += "\n".join(interaction_scripts)
+                rep["post"] += "\n".join(self._interaction_scripts)
+
+                self._interaction_scripts=[]    # reset the list !
 
             if next_js_call:
                 # if there was generator, set the next js call !
@@ -265,8 +275,9 @@ function action( o ) {
 
         except Exception as e:
             print("ERROR",e)
-            logger.error("Exception %s:\n%s", e, traceback.format_exc())
-            rep={"err": str(e) }
+            fullerror=traceback.format_exc()
+            logger.error("Exception %s:\n%s", e, fullerror)
+            rep={"err": fullerror if self.fullerror else str(e) }
 
         logger.info("RETURN --> %s",json.dumps(rep,indent=4))
 
@@ -298,12 +309,12 @@ function action( o ) {
         head=H.head()
         head <= H.meta(_charset="utf-8")
         head <= H.meta(_name="viewport",_content="width=device-width, initial-scale=1")
-        head <= H.meta(_name="version",_content=f"HTag {__version__}")
-        head <= H.title( self.title )
+        head <= H.meta(_name="version",_content=f"htag {__version__}")
         head <= self._statics
+        head <= H.title( self.title )   # set a default title
 
         body=H.body( "Loading...", _id=0 ) # IMPERSONATE (first interact on id #0)
-        return "<!DOCTYPE html>"+str(H.html( [head,body] ))
+        return "<!DOCTYPE html>"+str(H.html( head+body ))
 
     @property
     def title(self) -> str:
